// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.internal;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * With no arguments, enableassertions or -ea enables assertions. With one argument ending in "...", the switch enables assertions in the specified package and any subpackages. If the argument is simply "...", the switch enables assertions in the unnamed package in the current working directory. With one argument not ending in "...", the switch enables assertions in the specified class.
 * @author simon
 *
 */
public class ClassFileScanner {
	public interface Callback {
		public void doInMatch(String classFile);
	}

	public void scanAndMatch( Set<String> packageSpecs, Callback callback, File directory ) throws IOException {
		for( String packageArgument : packageSpecs ) {
			if( packageArgument.endsWith("...")){
				packageArgument = packageArgument.substring(0, packageArgument.length()-2);
			}
			
			StringTokenizer tokenizer = new StringTokenizer(packageArgument, ".", false);
			File subDirectory = new File(directory.getCanonicalPath());
			boolean fullMatch = true;
			StringBuilder packageNameBuilder = new StringBuilder();
			while( fullMatch && tokenizer.hasMoreTokens() ){
				String folderName = tokenizer.nextToken();
				packageNameBuilder.append(folderName);
				packageNameBuilder.append(".");
				File[] subFiles = subDirectory.listFiles();
				boolean partialMatch = false;
				if( subFiles != null ){
					for( File subFile : subFiles ){
						if( subFile.isDirectory() && subFile.getName().equals(folderName)){
							partialMatch = true;
							subDirectory = subFile;
							break;
						}
					}
				}
				if( !partialMatch ){
					fullMatch = false;
					break;
				}
			}
			if( fullMatch ){
				String packagePath = packageNameBuilder.toString();
				File[] classFiles = subDirectory.listFiles(new FilenameFilter() {
					public boolean accept(File dir, String name) {
						return name.endsWith(".class");
					}
				});
				for( File classFile : classFiles ){
					String className = classFile.getName();
					className = className.substring(0, className.lastIndexOf('.'));
					String canonicalName = packagePath+className;
					callback.doInMatch(canonicalName);
				}
			}
		}
		return;
	}

}
