// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.internal;

import java.util.ArrayList;
import java.util.List;

import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.NotFoundException;
import net.sf.chex4j.Contract;
import net.sf.chex4j.Post;
import net.sf.chex4j.Pre;

/*package*/class ClassMethodsInstrumentor extends MethodsInstrumentorBase
		implements ClassInstrumentor {

	public boolean instrumentClass(CtClass ctClass)
			throws ClassNotFoundException, CannotCompileException,
			NotFoundException {
		boolean workDone = false;

		workDone |= this.modifyClassMethods(ctClass);
		workDone |= this.modifyOverridesSuperClassMethods(ctClass);

		return workDone;
	}

	private boolean modifyOverridesSuperClassMethods(CtClass clazz)
			throws ClassNotFoundException, NotFoundException,
			CannotCompileException {
		CtClass ctClass = clazz;
		List<CtClass> superContracts = new ArrayList<CtClass>();

		while (ctClass.getSuperclass() != null) {
			ctClass = ctClass.getSuperclass();
			Contract c = (Contract) ctClass.getAnnotation(Contract.class);
			if (c != null) {
				superContracts.add(ctClass);
			}
		}
		List<MethodInstrumentionWork> work = new ArrayList<MethodInstrumentionWork>();
		for (CtClass superCtClass : superContracts) {
			for (CtMethod superMethod : superCtClass.getDeclaredMethods() ) {
				Post post = (Post) superMethod.getAnnotation(Post.class);
				Pre pre = (Pre) superMethod.getAnnotation(Pre.class);
				if (post != null || pre != null) {
					String methodName = superMethod.getName();
					String methodDescription = superMethod.getMethodInfo2().getDescriptor();
					CtMethod declaredMethod = clazz.getMethod(methodName, methodDescription);
					if (declaredMethod != null) {
						addSuperMethodWork(clazz, work, post, pre,
								declaredMethod);
					}
				}
			}
		}
		boolean workToDo = !work.isEmpty();
		if (workToDo) {
			performInstrumentationWork(work);
		}
		return workToDo;
	}

	private void addSuperMethodWork(CtClass clazz,
			List<MethodInstrumentionWork> work, Post post, Pre pre,
			CtMethod declaredMethod) {
		MethodInstrumentionWork w = new MethodInstrumentionWork();
		w.post = post;
		w.pre = pre;
		w.methodName = declaredMethod.getName();
		w.descriptor = declaredMethod.getMethodInfo2().getDescriptor();
		w.target = clazz;
		work.add(w);
	}

	private boolean modifyClassMethods(CtClass clazz)
			throws CannotCompileException, ClassNotFoundException,
			NotFoundException {

		List<MethodInstrumentionWork> work = new ArrayList<MethodInstrumentionWork>(
				clazz.getMethods().length);
		Contract c = (Contract) clazz.getAnnotation(Contract.class);
		if (c != null) {
			for (CtMethod m : clazz.getMethods()) {
				addWorkItem(clazz, work, m);
			}
			performInstrumentationWork(work);
		}
		return !work.isEmpty();
	}

}
