// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.internal;

import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.CtConstructor;
import net.sf.chex4j.Post;
import net.sf.chex4j.Pre;

/*package*/ class ConstructorInstrumentor extends ClassInstrumentorBase implements ClassInstrumentor {

	public boolean instrumentClass(CtClass ctClass)
			throws ClassNotFoundException, CannotCompileException {
		return this.modifyClassConstructors(ctClass);
	}

	private boolean modifyClassConstructors(CtClass ctClass)
			throws ClassNotFoundException, CannotCompileException {
		CtConstructor[] constructors = ctClass.getConstructors();
		boolean workDone = false;
		for (CtConstructor c : constructors) {
			workDone |= instrumentConstructor(c);
		}
		return workDone;
	}

	private boolean instrumentConstructor(CtConstructor c)
			throws ClassNotFoundException, CannotCompileException {
		Pre posti = (Pre) c.getAnnotation(Pre.class);
		if (posti != null) {
			throw new CannotCompileException(
					c.getLongName()
							+ " is a constructor may not be annotated with a @Post annotation as any super() in the constructor must be called before injected chex.");
		}

		boolean workDone = false;
		Post post = (Post) c.getAnnotation(Post.class);
		if (post != null ) {
			String preValue = post.value().trim();
			if( preValue.equals("") ){
				preValue = c.getName()+"$Post($$)";
			}
			String code = this.createCode(preValue, post.message());
			c.insertAfter(code);
			workDone = true;
		}
		return workDone;
	}
}
