// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.internal;

import java.util.List;

import net.sf.chex4j.Post;
import net.sf.chex4j.Pre;

import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.NotFoundException;
import javassist.bytecode.MethodInfo;

/*package*/ class MethodsInstrumentorBase extends ClassInstrumentorBase {

	/*package*/ static class MethodInstrumentionWork {
		Post post;
		Pre pre;
		String methodName;
		CtClass target;
		String descriptor;
	}
	
	public MethodsInstrumentorBase() {
		super();
	}

	protected void addWorkItem(CtClass clazz,
			List<MethodInstrumentionWork> work, CtMethod m)
			throws ClassNotFoundException {
		Post post = (Post) m.getAnnotation(Post.class);
		Pre pre = (Pre) m.getAnnotation(Pre.class);
		if (post != null || pre != null) {
			MethodInstrumentionWork w = new MethodInstrumentionWork();
			MethodInfo methodInfo = m.getMethodInfo2();
			w.post = post;
			w.pre = pre;
			w.methodName = m.getName();
			w.descriptor = methodInfo.getDescriptor();
			w.target = clazz;
			work.add(w);
		}
	}

	protected void performInstrumentationWork(List<MethodInstrumentionWork> work)
			throws NotFoundException, CannotCompileException {
		for (MethodInstrumentionWork w : work) {
			CtMethod m = w.target.getMethod(w.methodName, w.descriptor);
			CtClass declaringClass = m.getDeclaringClass();
			if( !declaringClass.isFrozen() ){
				if (w.post != null) {
					instrumentPost(m, w.post);
				}
				if (w.pre != null) {
					instrumentPre(m, w.pre);
				}
			}
		}
	}

	private void instrumentPre(CtMethod m, Pre pre)
			throws CannotCompileException {
				String preValue = pre.value();
				if( preValue.trim().equals("")) {
					preValue = m.getName() + "$Pre($$)";
				}
				String preMessage = pre.message();
				String code = createCode(preValue, preMessage);
				try {
					m.insertBefore(code);
				} catch (CannotCompileException e) {
					throw new CannotCompileException(code + "\nCaused by: "
							+ e.toString(), e);
				}
			}

	private void instrumentPost(CtMethod m, Post post)
			throws CannotCompileException {
				String postValue = post.value();
				if( postValue.trim().equals("")){
					postValue = m.getName() + "$Post($$)";
				}
				String postMessage = post.message();
				String code = createCode(postValue, postMessage);
				try {
					m.insertAfter(code);
				} catch (CannotCompileException e) {
					throw new CannotCompileException(code + "\nCaused by: "
							+ e.toString(), e);
				}
			}

}