// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.internal;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * This class is used to dump out intrumented classes to disk. This is helpful 
 * to create a test jar file to be deployed in a test build. 
 * @author simon
 *
 */
public class TransformedClassWriteToDiskListener implements TransformedClassListener {

	final private String dirPath;
	final private static String FILE_SEPARATOR = System.getProperty("file.separator");

	public TransformedClassWriteToDiskListener(String dirPath) {
		if( !dirPath.endsWith(FILE_SEPARATOR) ){
			dirPath += FILE_SEPARATOR;
		}
		this.dirPath = dirPath;
	}

	public void onClassTransformed(String className, byte[] classByteCode) {
		String fullPath = dirPath+className;
		try {
			String folderString = fullPath.substring(0, fullPath.lastIndexOf(FILE_SEPARATOR));
			File folder = new File(folderString);
			if( !folder.exists() ){
				folder.mkdirs();
			}
			File file = this.createFolderAndFile(fullPath);
			FileOutputStream fout = new FileOutputStream(file);
			fout.write(classByteCode);
			fout.flush();
			fout.close();
		} catch (Exception e) {
			throw new AssertionError("ERROR ERROR Cannot write out classfile to path "+fullPath+" "+e);
		} 
	}

	/*package*/ File createFolderAndFile(String fullPath) {
		StringTokenizer tokenizer = new StringTokenizer(fullPath, "/", false);
		List<String> pathParts = new ArrayList<String>();
		while(tokenizer.hasMoreTokens()){
			pathParts.add(tokenizer.nextToken());
		}
		StringBuilder buffer = new StringBuilder(fullPath.length());
		for( int i = 0; i < pathParts.size() - 1; i++ ){
			buffer.append(pathParts.get(i));
			buffer.append(FILE_SEPARATOR);
		}
		String fullPathToThisPart = buffer.toString();
		File folder = new File(fullPathToThisPart);
		if( !folder.exists() ){
			folder.mkdirs();
		}
		return new File(fullPath+".class");
	}

}
